import React, { ReactNode, useState } from "react";
import { parse_org } from "../../organic/target/wasm32-unknown-unknown/js/wasm";
import styles from "./OrgAst.module.css";

const OrgAst = (props: { setHighlight: Function; clearHighlights: Function; value: string }) => {
  const ast_tree = parse_org(props.value);
  console.log(JSON.stringify(ast_tree));

  function deselectAll() {
    props.clearHighlights();
  }

  if (ast_tree.status !== "success") {
    return <div className={styles.OrgAst}>Error! {ast_tree.content}</div>;
  } else {
    return (
      <div className={styles.OrgAst}>
        <OrgAstNode setHighlight={props.setHighlight} deselectAll={deselectAll} node={ast_tree.content} />
      </div>
    );
  }
};

const OrgAstNode = (props: { setHighlight: Function; deselectAll: Function; node: any }) => {

  const [selected, setSelected] = useState<boolean>(false);

  function selectNode() {
    props.setHighlight(props.node["standard-properties"]["begin"] - 1, props.node["standard-properties"]["end"] - 1);
    setSelected(true);
  }

  const nodeClassName = selected ? styles.OrgAstNode + " " + styles.selected : styles.OrgAstNode;

  return (
    <div className={nodeClassName}>
      <div className={styles.OrgAstNodeType} onClick={selectNode}>{props.node["ast-node"]}</div>
      <details>
        <summary>Standard Properties</summary>
        <dl>
          <dt>begin</dt>
          <dd>{props.node["standard-properties"]["begin"]}</dd>
          <dt>contents-begin</dt>
          <dd>{props.node["standard-properties"]["contents-begin"]}</dd>
          <dt>contents-end</dt>
          <dd>{props.node["standard-properties"]["contents-end"]}</dd>
          <dt>end</dt>
          <dd>{props.node["standard-properties"]["end"]}</dd>
          <dt>post-blank</dt>
          <dd>{props.node["standard-properties"]["post-blank"]}</dd>
        </dl>
      </details>
      {Array.isArray(props.node.children) && props.node.children.length > 0 ? (
        <details open={true}>
          <summary>Children</summary>
          <div className={styles.OrgAstChildren}>
            <OrgAstNodeList setHighlight={props.setHighlight} deselectAll={props.deselectAll} node_list={props.node.children} />
          </div>
        </details>
      ) : null}
    </div>
  );
};

const OrgAstNodeList = (props: { setHighlight: Function; deselectAll: Function; node_list: any[] }): React.JSX.Element[] => {
  return props.node_list.map((node) => {
    return <OrgAstNode setHighlight={props.setHighlight} deselectAll={props.deselectAll} node={node} />;
  });
};

export default OrgAst;
